#ifndef _OKC_COMM_H_
#define _OKC_COMM_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <errno.h>
#include <pthread.h>

#include "okc_types.h"
#include "okc_helper.h"


#ifdef __cplusplus
extern "C" {
#endif

#define OKC_BUFFER_SIZE 8192
#define OKC_ARBITRARY_STRING_LENGTH 256


#define OKC_QUIT 0xff
#define OKC_POSITION_CONTROL 0x11
#define OKC_AXIS_CONTROL 0x22

#define OKC_MIN_FD 3

okc_handle_t* 
okc_start_server (const char* host,const char* port, int mode);

int
okc_register_callback (okc_handle_t* okc, int robot_id, 
		       okc_callback callback, 
		       void* private_pointer_for_callback);

int
okc_stop_server (okc_handle_t* okc);

int
okc_is_robot_avail (okc_handle_t* okc, int robot_id);

int
okc_set_coords_correction (okc_handle_t* okc, int robot_id, 
			   coords k_correction);

int
okc_set_axis_correction (okc_handle_t* okc, int robot_id, 
			     lbr_axis a_correction);

int
okc_get_coords_act (okc_handle_t* okc, int robot_id, coords* k);

int
okc_get_coords_des (okc_handle_t* okc, int robot_id, coords* k);

int
okc_get_axis_act (okc_handle_t* okc, int robot_id, lbr_axis* l);

int
okc_get_axis_des (okc_handle_t* okc, int robot_id, lbr_axis* l);

int
okc_get_torques_axis_act (okc_handle_t* okc, int robot_id, lbr_axis* l);

int
okc_get_ft_tcp_est (okc_handle_t* okc, int robot_id, coords* k);

int
okc_get_status (okc_handle_t* okc, int robot_id, int* status);

int
okc_set_axis_stiffness (okc_handle_t* okc, int robot_id, 
			    lbr_axis a_stiff);

int
okc_set_axis_damping (okc_handle_t* okc, int robot_id, 
			  lbr_axis a_damp);

int
okc_set_cp_stiffness (okc_handle_t* okc, int robot_id, 
			  coords k_stiff);

int
okc_set_cp_damping (okc_handle_t* okc, int robot_id, 
			coords k_damp);

int
okc_switch_to_cp_impedance (okc_handle_t* okc, int robot_id);

int
okc_switch_to_axis_impedance (okc_handle_t* okc, int robot_id);

int
okc_switch_to_gravcomp (okc_handle_t* okc, int robot_id);

int
okc_switch_to_position (okc_handle_t* okc, int robot_id);

int
okc_send_break_to_robot (okc_handle_t* okc, int robot_id);

int
okc_send_drive_ptp_pos (okc_handle_t* okc, int robot_id, 
			coords position);

int
okc_send_drive_ptp_axis (okc_handle_t* okc, int robot_id, 
			 lbr_axis axis);

int
okc_set_desiredforce_params (okc_handle_t* okc, int robot_id, 
			     int forceprofile, int forcetiming, float amplitude,
			     float duration, float max_displacement,
			     float max_velocity, float profile_dependant1,
			     float profile_dependant2, float profile_dependant3);

int
okc_enable_desiredforce (okc_handle_t* okc, int robot_id, int direction);

int
okc_disable_desiredforce (okc_handle_t* okc, int robot_id);

#ifdef __cplusplus
}
#endif

#endif
