#ifndef _OKC_TYPES_H_
#define _OKC_TYPES_H_

#include <pthread.h>

#define OKC_MAX_ROBOTS 2
#define OKC_OK 0
#define OKC_UNAVAILABLE 2
#define OKC_ERROR -1
#define OKC_ACK_SET_CP_STIFFNESS 21
#define OKC_ACK_SET_CP_DAMPING 22
#define OKC_ACK_SET_AXIS_STIFFNESS 23
#define OKC_ACK_SET_AXIS_DAMPING 24
#define OKC_ACK_SET_SWITCH_CP_IMPEDANCE 25
#define OKC_ACK_SET_SWITCH_AXIS_IMPEDANCE 26
#define OKC_ACK_SET_SWITCH_GRAVCOMP 27
#define OKC_ACK_SET_SWITCH_POSITION 28
#define OKC_ACK_DRIVE_PTP_XYZ 31
#define OKC_ACK_DRIVE_PTP_AXIS 32
#define OKC_ACK_SET_DESIREDFORCE_PARAMS 33
#define OKC_ACK_SET_DESIREDFORCE_X 34
#define OKC_ACK_SET_DESIREDFORCE_Y 35
#define OKC_ACK_SET_DESIREDFORCE_Z 36
#define OKC_ACK_SWITCH_OFF_DESIREDFORCE 37

#define OKC_COMMAND_COMPLETE -222
#define OKC_COMMAND_PENDING -111
#define OKC_NORMAL_OP -1
#define OKC_SET_CP_STIFFNESS 1
#define OKC_SET_CP_DAMPING 2
#define OKC_SET_AXIS_STIFFNESS 3
#define OKC_SET_AXIS_DAMPING 4
#define OKC_SET_SWITCH_CP_IMPEDANCE 5
#define OKC_SET_SWITCH_AXIS_IMPEDANCE 6
#define OKC_SET_SWITCH_GRAVCOMP 7
#define OKC_SET_SWITCH_POSITION 8
#define OKC_SET_EXIT_GRACEFULLY 9
#define OKC_DRIVE_PTP 11
#define OKC_DRIVE_PTP_XYZ 11
#define OKC_DRIVE_PTP_AXIS 12
#define OKC_SET_DESIREDFORCE_PARAMS 13
#define OKC_SWITCH_ON_DESIREDFORCE 140
#define OKC_SET_DESIREDFORCE_X 14
#define OKC_SET_DESIREDFORCE_Y 15
#define OKC_SET_DESIREDFORCE_Z 16
#define OKC_SWITCH_OFF_DESIREDFORCE 17

#define OKC_SLOT_UNASSIGNED -1
#define OKC_SLOT_IN_USE 1

#define OKC_MODE_SETTER 0
#define OKC_MODE_CALLBACK_AXIS 1
#define OKC_MODE_CALLBACK_POS 2

#define OKC_NANOSLEEP_CYCLE 12000000

#define OKC_X_AXIS 1
#define OKC_Y_AXIS 2
#define OKC_Z_AXIS 3

#define OKC_DESIREDFORCE_NO 0
#define OKC_DESIREDFORCE_IMPULS 1
#define OKC_DESIREDFORCE_SINE 2

#define OKC_DESIREDFORCE_REQUIRE_DESIREDFORCESTART 1

#define OKC_DESIREDFORCE_SYNC 0
#define OKC_DESIREDFORCE_ASYNC 1


typedef struct Coords
{
  double x;
  double y;
  double z;
  double a;
  double b;
  double c;
} coords;

typedef struct Lbr_Axis
{
  double a1;
  double a2;
  double e1;
  double a3;
  double a4;
  double a5;
  double a6;
} lbr_axis;


typedef int (*okc_callback) (void*, double*, double*);


typedef struct Okc_Handle_T
{
  pthread_t* comm_thread;

  int flags;
  int listener;
  int mode;
  pthread_rwlock_t* callback_lock;
  okc_callback update_correction_callback[OKC_MAX_ROBOTS];
  void* private_to_update_correction_callback[OKC_MAX_ROBOTS];
  pthread_rwlock_t* robot_map_names_lock;
  int robot_map[OKC_MAX_ROBOTS];
  char* robot_names[OKC_MAX_ROBOTS];
  pthread_rwlock_t* pos_axis_in_lock;
  coords* pos_correction;
  lbr_axis* axis_correction;
  int command[OKC_MAX_ROBOTS];
  int subcommand[OKC_MAX_ROBOTS];
  int forceprofile[OKC_MAX_ROBOTS];
  int forceimmediate[OKC_MAX_ROBOTS];
  int forcetiming[OKC_MAX_ROBOTS];
  double** params;
  int send_break[OKC_MAX_ROBOTS];
  pthread_rwlock_t* rist_lbra_out_lock;
  coords* pos_act;
  coords* pos_des;
  lbr_axis* axis_act;
  lbr_axis* axis_des;
  int status[OKC_MAX_ROBOTS];
  long int update_ticks[OKC_MAX_ROBOTS];
  int pending_command[OKC_MAX_ROBOTS];
  int ticks_command[OKC_MAX_ROBOTS];
  lbr_axis* torques_axis_act;
  coords* ft_tcp_est;
} okc_handle_t;


#endif
